/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */

//ref https://github.com/bahmutov/cache-require-paths
module.exports = function (cacheFile) {
	//モジュールを参照
	var Module = require("module");

	var fs = require("fs");
	var originRequire = Module.prototype.require;

	//キャッシュを参照
	var nameCache;
	try {
		var stat = fs.statSync(cacheFile);
		if (stat.isFile()) {
			nameCache = JSON.parse(fs.readFileSync(cacheFile, "utf8"));
		}
	} catch (err) {
		nameCache = {};
	}

	//オリジナルのrequireをフック
	Module.prototype.require = function (name) {
		var currentCache;
		var pathToLoad;

		//現在のモジュールに対するキャッシュを参照
		currentCache = nameCache[this.filename];
		//なければ保存
		if (!currentCache) {
			currentCache = {};
			nameCache[this.filename] = currentCache;
		}

		//キャッシュを参照
		if (currentCache[name] && typeof currentCache[name] === "string") {
			//解決済み
			pathToLoad = currentCache[name];
		} else {
			//解決して保存
			pathToLoad = Module._resolveFilename(name, this);
			currentCache[name] = pathToLoad;
		}

		try {
			//ロード
			return originRequire.call(this, pathToLoad);
		} catch (err) {
			//再度解決
			pathToLoad = Module._resolveFilename(name, this);
			currentCache[name] = pathToLoad;
			return originRequire.call(this, pathToLoad);
		}
	};

	//終了時にキャッシュ保存
	process.once("exit", function () {
		try {
			fs.writeFileSync(cacheFile, JSON.stringify(nameCache, null, "\t"), "utf8");
		} catch (err) {
			console.error("Failed saving cache: " + err.toString());
		}
	});
};
