/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */

"use strict";

var readline = require("readline");
var util = require("./util");
var gulp = require("gulp");

module.exports = function (commands, run) {
	commands = commands || {};

	if (!commands.prod) {
		commands.prod = function () {
			util.restart("production");
		};
	}
	if (!commands.stag) {
		commands.stag = function () {
			util.restart("staging");
		};
	}
	if (!commands.dev) {
		commands.dev = function () {
			util.restart("development");
		};
	}
	if (!commands.exit) {
		commands.exit = function () {
			process.exit(0);
		};
	}
	if (!commands.restart) {
		commands.restart = function () {
			util.restart();
		};
	}

	var cli = readline.createInterface({
		input: process.stdin,
		output: process.stdout,
		completer: line => {
			var completions = Object.keys(gulp.tasks).concat(Object.keys(commands));
			var hits = completions.filter((c) => c.indexOf(line) == 0);
			return [hits, line];
		},
	});
	cli.on("SIGINT", _ => process.exit(0));

	var prompt = _ => {
		cli.question("> ", line => {
			line = line.trim();
			if (line && (line in gulp.tasks)) {
				run(line);
			} else if (line in commands) {
				commands[line]();
			} else if (line) {
				console.log("Command:" + line + " not found.");
			}

			prompt();
		});
	}

	prompt();
}
