/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */

"use strict";

var path = require("path");
var util = require("./util");
var loader = require("./loader");
var Expression = require("./expression");
var env = require("./env");
var version = require("./version");

//式と変数の適用
function applyExpression(config, vars) {
	if (util.isArray(config)) {
		for (let i = 0, l = config.length; i < l; i++) {
			let val = config[i];
			if (util.isString(val) && val.startsWith("expr:")) {
				config[i] = new Expression(val.substr(5)).eval(vars);
			} else if (util.isArray(val) || util.isObject(val)) {
				applyExpression(val, vars);
			}
		}
	} else if (util.isObject(config)) {
		for (let key in config) {
			let val = config[key];
			if (util.isString(val) && val.startsWith("expr:")) {
				config[key] = new Expression(val.substr(5)).eval(vars);
			} else if (util.isArray(val) || util.isObject(val)) {
				applyExpression(val, vars);
			}
		}
	}
}

//読み込み済みの設定
var loadedConfig = {};

//読み込み
exports.load = function load(pathname) {
	//ベースディレクトリを補完
	var base = util.getBase();
	if (base) {
		pathname = path.join(base, pathname);
	}

	//基本設定読み込み
	var config = loader.loadObject(pathname);
	if (!util.isObject(config)) {
		config = {};
	}
	var override = null;

	//envに応じたファイルを検索
	try {
		override = loader.loadObject(pathname + "." + env);
		if (!util.isObject(override)) {
			override = {};
		}
	} catch (e) {
	}

	//上書き可能なものがあればマージ
	if (override) {
		config = util.mergeJson(config, override);
	}

	loadedConfig = config;
}

//設定取得
exports.get = function get(taskpath) {
	//共通設定を取得
	var shared = loadedConfig._;
	if (!util.isObject(shared)) {
		shared = {};
	}

	//固有設定を取得
	var taskConfig = loadedConfig;
	for (var i = 0; i < taskpath.length; i++) {
		taskConfig = taskConfig[taskpath[i]];
		if (!util.isObject(taskConfig)) {
			taskConfig = {};
		}
	}

	//共通設定に固有設定をマージする
	taskConfig = util.mergeJson(shared, taskConfig);

	//変数作成
	var vars = loadedConfig.$;
	if (!util.isObject(vars)) {
		vars = {};
	}

	//基本変数に設定
	if (!("version" in vars)) {
		vars.version = version;
	}
	if (!("env" in vars)) {
		vars.env = env;
	}
	if (!("year" in vars)) {
		vars.year = function(from){
			var thisYear = new Date().getFullYear();

			if(thisYear == from){
				return String(thisYear);
			}

			return from + "-" + thisYear;
		};
	}

	//変数展開
	applyExpression(taskConfig, vars);
	return taskConfig;
}
