/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */

"use strict";

var pattern = /\{{1,2}|\}{1,2}/g;
var evalFunction = new Function("__expr__", "with(this){return eval(__expr__)}");

//テキスト
class TextValue {

	constructor(buf) {
		this.buf = buf;
	}

	eval(context) {
		return this.buf;
	}
}

//式
class ExprValue {

	constructor(buf) {
		this.buf = buf;
	}

	eval(context) {
		try {
			return evalFunction.call(context, this.buf);
		} catch (e) {
			return "{" + e.toString() + "}";
		}
	}
}

//式オブジェクト
module.exports = class Expression {

	constructor(str) {
		this.values = [];
		this.origin = str;
		this.error = null;

		this._compile();
	}

	eval(context) {
		if (this.error) {
			return this.error;
		}

		var result = "";

		for (let value of this.values) {
			result += value.eval(context);
		}

		return result;
	}

	_compile() {
		var tokens = [];
		var last = 0;
		var str = this.origin;

		str.replace(pattern, function (match, start) {
			if (last != start) {
				tokens[tokens.length] = {
					str: str.substring(last, start),
					begin: last,
					end: start,
				};
			}
			tokens[tokens.length] = {
				str: match,
				begin: start,
				end: start + match.length,
			};
			last = start + match.length;
			return "";
		});

		if (last != str.length) {
			tokens[tokens.length] = {
				str: str.substring(last),
				begin: last,
				end: str.length,
			};
		}

		var mode = "text";
		var buf = "";
		for (let token of tokens) {
			switch (token.str) {
				case "{":
					if (mode == "expr") {
						this.error = `already started expr in [${str}] in (${token.begin})`;
						return;
					}
					if (buf) {
						this.values[this.values.length] = new TextValue(buf);
						buf = "";
					}
					mode = "expr";
					break;
				case "{{":
					buf += "{";
					break;
				case "}":
					if (mode == "text") {
						this.error = `not started expr in [${str}] in (${token.begin})`;
						return;
					}
					if (!buf) {
						this.error = `expr is empty in [${str}] in (${token.begin})`;
						return;
					}
					this.values[this.values.length] = new ExprValue(buf);
					buf = "";
					mode = "text";
					break;
				case "}}":
					buf += "}";
					break;
				default:
					buf += token.str;
					break;
			}
		}
		if (mode == "expr") {
			this.error = `not closed expr in [${str}]`;
			return;
		}
		if (buf) {
			this.values[this.values.length] = new TextValue(buf);
		}
	}
}
