/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */
"use strict";

//gulpから起動されたか
var bootFromGulp = (function () {
	var path = require("path");
	var base = path.basename(process.argv[1]);
	if (base == "gulp.js" || base == "gulp") {
		return true;
	}

	return false;
})();

//起動を要求
function requestStart() {
	if (!bootFromGulp) {
		require("./util").start();
	}
}

//基本関数
var gulpis = {

	//プロジェクトベース
	get base() {
		requestStart();
		return require("./util").getBase();
	},

	//ルートディレクトリ
	get root() {
		requestStart();
		return require("./util").cwd;
	},

	//モジュールロードのキャッシュ
	cache(filename) {
		requestStart();

		if (bootFromGulp) {
			require("./cache")(filename);
		}
	},

	//タスク
	get Task() {
		requestStart();

		if (bootFromGulp) {
			return require("./task");
		} else {
			return function () { };
		}
	},

	//設定の読み込み
	get config() {
		requestStart();

		if (bootFromGulp) {
			return require("./config").load;
		} else {
			return function () { };
		}
	},

	//タスクの取り込み
	get register() {
		requestStart();

		if (bootFromGulp) {
			return require("./registory");
		} else {
			return function () { };
		}
	},

	//NODE_ENV
	get env() {
		requestStart();
		return require("./env");
	},

	//オプション
	get options() {
		requestStart();
		return require("./options");
	},

	//バージョン
	get version() {
		requestStart();
		return require("./version");;
	},

	//gulp
	get gulp() {
		requestStart();
		return require("gulp");
	},

	//自分自身の動作情報表示
	about() {
		requestStart();

		if (bootFromGulp) {
			console.log(require("./util").about());
		}
	},
};

if (bootFromGulp) {
	//gulpから起動された場合
	//起点のファイルも監視対象に
	require("./loader").watchBootstrap();
}

//ショートカット関数
var gulpisFn = function (options) {
	if (!options) {
		return;
	}

	if (options.cache) {
		gulpis.cache(options.cache);
	}
	if (options.config) {
		gulpis.config(options.config);
	}
	if (options.register) {
		let paths = Array.isArray(options.register) ? options.register : [options.register];
		for (let path of paths) {
			gulpis.register(path);
		}
	}

	gulpis.about();
};

//プロトタイプ拡張
Object.setPrototypeOf(gulpisFn, gulpis);

module.exports = gulpisFn;
