/*!
 * gulpis
 *
 * Copyright (c) 2016 Katsunori Koyanagi.
 * Licensed under the MIT license.
 */

"use strict";

var path = require("path");
var fs = require("fs");
var util = require("./util");

//package.jsonによる情報
var packageVersion = new class {

    constructor() {
        this.info = null;
    }

    get name() {
        this.init();
        return this.info.name;
    }

    get author() {
        this.init();
        return this.info.author;
    }

    get version() {
        this.init();
        return this.info.version;
    }

    get semver() {
        this.init();
        return this.info.semver;
    }

    get major() {
        this.init();
        return this.info.major;
    }

    get minor() {
        this.init();
        return this.info.minor;
    }

    get patch() {
        this.init();
        return this.info.patch;
    }

    get release() {
        this.init();
        return this.info.release;
    }

    get build() {
        this.init();
        return this.info.build;
    }

    init() {
        if (this.info != null) {
            return;
        }

        //package.json取得
        var pathname = require.resolve(path.join(util.cwd, "package.json"));
        var packageInfo = require(pathname);

        //変更監視
        var watcher = fs.watch(pathname, { persistent: false }, _ => {
            this.info = null;
            delete require.cache[pathname];
            watcher.close();
        });

        var name = packageInfo.name;
        var author = packageInfo.author;
        var version = packageInfo.version;

        //from https://github.com/coolaj86/semver-utils/blob/master/semver-utils.js
        var pattern = /^v?((\d+)\.(\d+)\.(\d+))(?:-([\dA-Za-z\-]+(?:\.[\dA-Za-z\-]+)*))?(?:\+([\dA-Za-z\-]+(?:\.[\dA-Za-z\-]+)*))?$/;
        var m = pattern.exec(version) || [];

        this.info = {
            author: author,
            name: name,
            semver: m[0] || "",
            version: m[1] || "",
            major: parseInt(m[2], 10) || 0,
            minor: parseInt(m[3], 10) || 0,
            patch: parseInt(m[4], 10) || 0,
            release: m[5] || "",
            build: m[6] || "",
        };
    }
}

module.exports = {

    get name() {
        return packageVersion.name;
    },

    get author() {
        return packageVersion.author;
    },

    get version() {
        return packageVersion.version;
    },

    get semver() {
        return packageVersion.semver;
    },

    get major() {
        return packageVersion.major;
    },

    get minor() {
        return packageVersion.minor;
    },

    get patch() {
        return packageVersion.patch;
    },

    get release() {
        return packageVersion.release;
    },

    get build() {
        return packageVersion.build;
    },

    //文字列表記
    toString() {
        return this.semver;
    },

    //文字列表記
    toStringWithDateTime() {
        var date = new Date();
        var datetime = "";

        datetime += date.getFullYear();
        datetime += ("0" + (date.getMonth() + 1)).substr(-2);
        datetime += ("0" + date.getDate()).substr(-2);
        datetime += ("0" + date.getHours()).substr(-2);
        datetime += ("0" + date.getMinutes()).substr(-2);
        datetime += ("0" + date.getSeconds()).substr(-2);

        return this.semver + "@" + datetime;
    },
};
